<?php

declare(strict_types=1);

namespace Drupal\file_visibility_track_usage;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DependencyInjection\ServiceModifierInterface;
use Drupal\file_visibility\FileVisibility;
use Drupal\track_usage\Recorder;

/**
 * Makes the order of destruct sequences predictable.
 */
class FileVisibilityTrackUsageServiceProvider implements ServiceModifierInterface {

  /**
   * {@inheritdoc}
   */
  public function alter(ContainerBuilder $container): void {
    $recorder = $container->getDefinition(Recorder::class);
    $recorderPriority = $recorder->getTag('needs_destruction')[0]['priority'] ?? 0;

    // The 'Drupal\track_usage\Recorder' service destructor always takes
    // precedence over 'Drupal\file_visibility\FileVisibility' destructor.
    $fileVisibility = $container->getDefinition(FileVisibility::class);
    $tags = $fileVisibility->getTags();
    $tags['needs_destruction'][0]['priority'] = $recorderPriority - 20;
    $fileVisibility->setTags($tags);
  }

}
