<?php

declare(strict_types=1);

namespace Drupal\file_visibility\Controller;

use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\file_visibility\FileVisibilityInterface;
use Drupal\system\FileDownloadController;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Replacement for \Drupal\system\FileDownloadController.
 *
 * The page displayed after the file visibility has been changed to public, may
 * still render file links with previous private URL. This happens because the
 * move from private to public is performed in the kernel termination phase,
 * after the response is sent back to the client. We wrap the original private
 * file controller and, if we detect such a situation, we return a redirect to
 * the new public location of the file.
 */
class FileVisibilityDownloadController extends FileDownloadController {

  public function __construct(
    StreamWrapperManagerInterface $streamWrapperManager,
    protected readonly FileUrlGeneratorInterface $urlGenerator,
  ) {
    parent::__construct($streamWrapperManager);
  }

  /**
   * {@inheritdoc}
   */
  public function download(Request $request, $scheme = 'private'): BinaryFileResponse|RedirectResponse {
    $target = $request->query->get('file');

    // Remove private:// from the beginning.
    $prefix = substr(FileVisibilityInterface::PRIVATE_LOCATION, 10);
    if (str_starts_with($target, $prefix)) {
      $target = substr($target, strlen($prefix));
      $uri = $this->streamWrapperManager->normalizeUri("public://$target");
      if (file_exists($uri)) {
        return new RedirectResponse($this->urlGenerator->generate($uri)->toString());
      }
    }
    return parent::download($request, $scheme);
  }

}
