<?php

declare(strict_types=1);

namespace Drupal\file_visibility\Hook;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\file_visibility\FileVisibilityInterface;

/**
 * Entity operations hooks.
 */
class EntityOperationHooks {

  public function __construct(
    protected readonly FileVisibilityInterface $fileVisibility,
  ) {}

  /**
   * Updates field visibility on entity CRUD.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity instance.
   */
  #[Hook('entity_insert')]
  #[Hook('entity_update')]
  #[Hook('entity_delete')]
  #[Hook('entity_translation_delete')]
  #[Hook('entity_revision_delete')]
  public function updateFileVisibility(EntityInterface $entity): void {
    $this->fileVisibility->updateFilesVisibility($entity);
  }

}
